DROP VIEW IF EXISTS sod_case_action_v;
DROP VIEW IF EXISTS spermission_v;
DROP VIEW IF EXISTS suser_name_v;

CREATE VIEW suser_name_v AS 
  SELECT 
    unique_user_id, 
    display_name 
  FROM suser 
  WHERE effective = true and deleted = false;

CREATE VIEW spermission_v AS 
 SELECT DISTINCT spermission.id,
    spermission.account_grant,
    spermission.assignable,
    spermission.authoritative,
    spermission.bound_by_id,
    spermission.boundable,
    spermission.cost,
    spermission.deleted,
    spermission.description,
    spermission.display_name,
    spermission.driver_id,
    spermission.effective,
    spermission.exclude_from_catalog,
    spermission.is_curated,
    spermission.logical_id,
    spermission.merged,
    spermission.name,
    spermission.not_reviewable,
    spermission.permission_id,
    spermission.prov_attribute,
    spermission.prov_by_unique_permission_id,
    spermission.prov_native_value,
    spermission.prov_unique_driver_id,
    spermission.provisionable,
    spermission.risk,
    spermission.type,
    spermission.unique_bound_by_id,
    spermission.unique_permission_id,
    spermission.value,
    spermission.value_type,
    spermission.application,
    spermission.base_application,
    spermission.collected_entity,
    spermission.data_collection,
    spermission.effective_entity,
    spermission.sversion,
    spermission.auto_generated,
    spermission.prov_unique_permission_id,
    auth_by.brole_name AS authorized_by,
    string_agg(su.display_name::text, ' | '::text) AS perm_owner_name
   FROM spermission
     LEFT JOIN ( SELECT brole.brole_name,
            brole.unique_item_id
           FROM ( SELECT br_incl_item.business_role_id,
                    br_incl_item.unique_item_id,
                    ( SELECT DISTINCT brole_1.name
                           FROM business_role brole_1
                          WHERE br_incl_item.business_role_id = brole_1.id AND brole_1.approval_state::text = 'PUBLISHED'::text) AS brole_name
                   FROM business_role_incl_item br_incl_item) brole,
            spermission spermission_1
          WHERE brole.unique_item_id::text = spermission_1.unique_permission_id::text AND brole.brole_name IS NOT NULL) auth_by ON auth_by.unique_item_id::text = spermission.unique_permission_id::text
     LEFT JOIN resolved_spermission_owner rso ON rso.permission = spermission.id
     LEFT JOIN suser_name_v su ON su.unique_user_id::text = rso.owner_unique_id::text
  WHERE spermission.deleted = false AND spermission.effective = true
  GROUP BY spermission.id, spermission.account_grant, spermission.assignable, spermission.authoritative, spermission.bound_by_id, spermission.boundable, spermission.cost, spermission.deleted, spermission.description, spermission.display_name, spermission.driver_id, spermission.effective, spermission.exclude_from_catalog, spermission.is_curated, spermission.logical_id, spermission.merged, spermission.name, spermission.not_reviewable, spermission.permission_id, spermission.prov_attribute, spermission.prov_by_unique_permission_id, spermission.prov_native_value, spermission.prov_unique_driver_id, spermission.provisionable, spermission.risk, spermission.type, spermission.unique_bound_by_id, spermission.unique_permission_id, spermission.value, spermission.value_type, spermission.application, spermission.base_application, spermission.collected_entity, spermission.data_collection, spermission.effective_entity, spermission.sversion, spermission.auto_generated, spermission.prov_unique_permission_id, auth_by.brole_name
  ORDER BY spermission.name;

CREATE VIEW sod_case_action_v AS 
 SELECT sod_case_info.id AS case_number,
    sod_case_info.state AS case_status,
    sod_case_info.policy,
    sod_case_info.policy_name,
    sod_policy.deleted AS policy_status,
    sod_case_info.violator_is_user,
    sod_case_info.violator_name,
    sod_case_info.violator_title,
    sod_case_action.id AS action_id,
    sod_case_action.action,
    sod_case_action.action_comment,
    sod_case_action.action_time,
    suser.display_name AS performed_by,
    sod_case_action.control_period,
        CASE
            WHEN sod_case_action.control_period IS NOT NULL THEN sod_case_action.action_time + (( SELECT (sod_case_action.control_period || ' days'::text)::interval AS "interval"))
            ELSE NULL::timestamp with time zone
        END AS approval_expiration,
    sod_removal_item.type AS item_type,
        CASE
            WHEN sod_removal_item.type::text = 'PERMISSION'::text THEN spermission_v.name
            ELSE role_policy.name
        END AS item_name,
    count(cri.id) AS total_number,
    ( SELECT count(*) AS count
           FROM sod_case_action sca2
             LEFT JOIN change_request_item cri_1 ON cri_1.sod_case_action = sod_case_action.id
             LEFT JOIN change_item_action cia ON cri_1.latest_action = cia.id
          WHERE sca2.id = sod_case_action.id AND cia.change_item_status::text = 'PENDING'::text AND cri_1.provisioning_type::text = 'MANUAL'::text) AS pending_manual_number,
    ( SELECT count(*) AS count
           FROM sod_case_action sca2
             LEFT JOIN change_request_item cri_1 ON cri_1.sod_case_action = sod_case_action.id
             LEFT JOIN change_item_action cia ON cri_1.latest_action = cia.id
          WHERE sca2.id = sod_case_action.id AND cia.change_item_status::text = 'PENDING'::text AND cri_1.provisioning_type::text = 'IDENTITY_MANAGER'::text) AS pending_idm_number,
    ( SELECT count(*) AS count
           FROM sod_case_action sca2
             LEFT JOIN change_request_item cri_1 ON cri_1.sod_case_action = sod_case_action.id
             LEFT JOIN change_item_action cia ON cri_1.latest_action = cia.id
          WHERE sca2.id = sod_case_action.id AND cia.change_item_status::text = 'PENDING'::text AND cri_1.provisioning_type::text = 'EXTERNAL_WORKFLOW'::text) AS pending_ext_number,
    ( SELECT count(*) AS count
           FROM sod_case_action sca2
             LEFT JOIN change_request_item cri_1 ON cri_1.sod_case_action = sod_case_action.id
             LEFT JOIN change_item_action cia ON cri_1.latest_action = cia.id
          WHERE sca2.id = sod_case_action.id AND cia.change_item_status::text = 'INITIALIZED'::text) AS initialized_number,
    ( SELECT count(*) AS count
           FROM sod_case_action sca2
             LEFT JOIN change_request_item cri_1 ON cri_1.sod_case_action = sod_case_action.id
             LEFT JOIN change_item_action cia ON cri_1.latest_action = cia.id
          WHERE sca2.id = sod_case_action.id AND cia.change_item_status::text = 'VERIFIED'::text) AS verified_number,
    ( SELECT count(*) AS count
           FROM sod_case_action sca2
             LEFT JOIN change_request_item cri_1 ON cri_1.sod_case_action = sod_case_action.id
             LEFT JOIN change_item_action cia ON cri_1.latest_action = cia.id
          WHERE sca2.id = sod_case_action.id AND cia.change_item_status::text = 'REFUSED'::text) AS refused_number,
    ( SELECT count(*) AS count
           FROM sod_case_action sca2
             LEFT JOIN change_request_item cri_1 ON cri_1.sod_case_action = sod_case_action.id
             LEFT JOIN change_item_action cia ON cri_1.latest_action = cia.id
          WHERE sca2.id = sod_case_action.id AND cia.change_item_status::text = 'VERIFICATION_IGNORED'::text) AS verification_ignored_number,
    ( SELECT count(*) AS count
           FROM sod_case_action sca2
             LEFT JOIN change_request_item cri_1 ON cri_1.sod_case_action = sod_case_action.id
             LEFT JOIN change_item_action cia ON cri_1.latest_action = cia.id
          WHERE sca2.id = sod_case_action.id AND cia.change_item_status::text = 'NOT_VERIFIED'::text) AS not_verified_number,
    ( SELECT count(*) AS count
           FROM sod_case_action sca2
             LEFT JOIN change_request_item cri_1 ON cri_1.sod_case_action = sod_case_action.id
             LEFT JOIN change_item_action cia ON cri_1.latest_action = cia.id
          WHERE sca2.id = sod_case_action.id AND cia.change_item_status::text = 'VERIFICATION_TIMEOUT'::text) AS verification_timeout_number,
    ( SELECT count(*) AS count
           FROM sod_case_action sca2
             LEFT JOIN change_request_item cri_1 ON cri_1.sod_case_action = sod_case_action.id
             LEFT JOIN change_item_action cia ON cri_1.latest_action = cia.id
          WHERE sca2.id = sod_case_action.id AND cia.change_item_status::text = 'ERROR'::text) AS error_number,
    ( SELECT count(*) AS count
           FROM sod_case_action sca2
             LEFT JOIN change_request_item cri_1 ON cri_1.sod_case_action = sod_case_action.id
             LEFT JOIN change_item_action cia ON cri_1.latest_action = cia.id
          WHERE sca2.id = sod_case_action.id AND cia.change_item_status::text = 'COMPLETED'::text AND cri_1.provisioning_type::text = 'MANUAL'::text) AS completed_manual_number,
    ( SELECT count(*) AS count
           FROM sod_case_action sca2
             LEFT JOIN change_request_item cri_1 ON cri_1.sod_case_action = sod_case_action.id
             LEFT JOIN change_item_action cia ON cri_1.latest_action = cia.id
          WHERE sca2.id = sod_case_action.id AND cia.change_item_status::text = 'COMPLETED'::text AND cri_1.provisioning_type::text = 'IDENTITY_MANAGER'::text) AS completed_idm_number,
    ( SELECT count(*) AS count
           FROM sod_case_action sca2
             LEFT JOIN change_request_item cri_1 ON cri_1.sod_case_action = sod_case_action.id
             LEFT JOIN change_item_action cia ON cri_1.latest_action = cia.id
          WHERE sca2.id = sod_case_action.id AND cia.change_item_status::text = 'COMPLETED'::text AND cri_1.provisioning_type::text = 'EXTERNAL_WORKFLOW'::text) AS completed_ext_number,
    ( SELECT count(*) AS count
           FROM sod_case_action sca2
             LEFT JOIN change_request_item cri_1 ON cri_1.sod_case_action = sod_case_action.id
             LEFT JOIN change_item_action cia ON cri_1.latest_action = cia.id
          WHERE sca2.id = sod_case_action.id AND cia.change_item_status::text = 'IN_PROGRESS'::text) AS in_progress_number
   FROM sod_case_info
     JOIN sod_policy ON sod_policy.id = sod_case_info.policy
     JOIN sod_case_action ON sod_case_action.sod_case = sod_case_info.id
     LEFT JOIN sod_removal_item ON sod_removal_item.sod_case_action = sod_case_action.id
     LEFT JOIN spermission_v ON spermission_v.unique_permission_id::text = sod_removal_item.unique_id::text
     LEFT JOIN role_policy ON role_policy.unique_policy_id::text = sod_removal_item.unique_id::text AND role_policy.deleted = false
     LEFT JOIN suser ON sod_case_action.performed_by::text = suser.unique_user_id::text AND suser.deleted = false AND suser.effective = true
     LEFT JOIN change_request_item cri ON cri.sod_case_action = sod_case_action.id
  GROUP BY sod_case_info.id, sod_case_info.state, sod_case_info.policy, sod_case_info.policy_name, sod_policy.deleted, sod_case_info.violator_is_user, sod_case_info.violator_name, sod_case_info.violator_title, sod_case_action.id, sod_case_action.action, sod_case_action.action_comment, sod_case_action.action_time, suser.display_name, sod_case_action.control_period, spermission_v.name, role_policy.name, sod_removal_item.type
  ORDER BY sod_case_info.policy_name, sod_case_info.policy, sod_case_info.violator_name, sod_case_action.action_time;

ALTER TABLE suser_name_v
  OWNER TO igops;
GRANT ALL ON TABLE suser_name_v TO igops;
GRANT SELECT ON TABLE suser_name_v TO ig_report_role;

 ALTER TABLE spermission_v
   OWNER TO igops;
 GRANT ALL ON TABLE spermission_v TO igops;
 GRANT SELECT ON TABLE spermission_v TO ig_report_role;

 ALTER TABLE sod_case_action_v
  OWNER TO igops;
GRANT ALL ON TABLE sod_case_action_v TO igops;
GRANT SELECT ON TABLE sod_case_action_v TO ig_report_role;
